#-----------------------------------------------------------------------------
# Gretel
# Copyright (c) 2004 David G. Drumright (Ockham)
# Oct 7, 2004

# New Feb 28, 2008
# New version with custom scaled pointer, so you
# can move the grid to various angles.
# The pointer moves on the grid identically to the
# drawn canvas.
#-----------------------------------------------------------------------------

from Tkinter import *
import tkFileDialog
import poser
import string
scene = poser.Scene()
import pickle
import os

#-----------------------------------------------------------------------------

global MainRecord
# Load the custom pointer prop.
SelItem=None
PropName=os.path.normpath(os.path.join('runtime','libraries','props','Tiler','GretelPointer.pp2'))

try:
	scene.LoadLibraryProp(PropName)
except:
	raise "Prop not usable. Try again."

# Go for the latest-loaded pointer, so you can run
# Gretel repeatedly to make mirrored or differently-scaled motions.
Pointers=[]
for A in scene.Actors():
	if string.count(A.Name(),'pointer'):
		Pointers.append(A.Name())
Pointers.sort()
SelItem=scene.Actor( Pointers[-1] ) # Take the highest-numbered one.

#-----------------------------------------------------------------------------

class App:
	def __init__(self, master, textMessage):
		self.master = master
		master.title("Gretel")

		self.TopFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
		self.TopFrame.grid(row=1,column=0)
		self.BotFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
		self.BotFrame.grid(row=3,column=0)

		self.StatusEntry = Entry(self.TopFrame,width=70)
		self.StatusEntry.grid(row=0,column=0,columnspan=3)
		self.StatusEntry.insert(0,"First pick a DAT file made by Hansel.")

		# Go/nogo buttons.
		self.ButtonName = Button(self.TopFrame, text="Pick file", command=self.HandleName)
		self.ButtonName.grid(row=1, column=0,sticky=E+W)

		Label(self.TopFrame, text="Start frame:" ).grid(row=2, sticky=W)
		self.StartEntry = Entry(self.TopFrame, width=5)
		self.StartEntry.grid(row=2, column=1)
		self.StartEntry.insert(0,str(1))

		Label(self.TopFrame, text="End frame:" ).grid(row=3, sticky=W)
		self.EndEntry = Entry(self.TopFrame, width=5)
		self.EndEntry.grid(row=3, column=1)
		self.EndEntry.insert(0,str(scene.NumFrames()))

		self.ButtonGo = Button(self.BotFrame, text="Go", command=self.HandleGo)
		self.ButtonGo.grid(row=0, column=0)
		self.ButtonCancel = Button(self.BotFrame, text="Cancel", command=self.HandleCancel)
		self.ButtonCancel.grid(row=0, column=2)

		self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

	# - - - - - - - - - - - - - - - - - -

	def ShowStatus(self,S): # Just saving repetition
		self.StatusEntry.delete(0,END)
		self.StatusEntry.insert(0,S)
		self.StatusEntry.update_idletasks()

	# - - - - - - - - - - - - - - - - - -
	def HandleName(self):
		global FullFileName
		global MainRecord


		# Pop up file box to get name.
		BasePathUncut=poser.AppLocation()
		BasePath=BasePathUncut[:(-len('poser.exe'))]
		MyPath=os.path.normpath(BasePath+'/runtime/*.DAT') # Use this for release
		fn=tkFileDialog.askopenfilename(initialfile=MyPath, filetypes=[("DAT file", "*.DAT *.dat")] )

		if not fn:
			self.ShowStatus("No file selected. Try again.")
			return 0

		# Open the file
		FullFileName=os.path.abspath(fn)
		fp=open(FullFileName,'r')
		MainRecord=pickle.load(fp)
		fp.close()
		self.ShowStatus("Now set the start and end frames and Lift, then hit Go.")

	# - - - - - - - - - - - - - - - - - -

	def HandleGo(self):
		global FullFileName
		global MainRecord
		# Convert the time.  The DAT file is in ms, so we need a factor
		# to convert ms to frames.	We have the desired total frames.
		# The total ms is the time part of the last thing in the array.
		SF=self.StartEntry.get()
		if SF: StartFrame=int(SF)
		else:  StartFrame=1
		EF=self.EndEntry.get()
		if EF: EndFrame=int(EF)
		else:  EndFrame=scene.NumFrames()
		TotalFrames=EndFrame-StartFrame+1


		# We need to do MS * Factor = Frame, so Factor must be Frame/MS.
		(TotalMS,X,Y,Status)=MainRecord[-1]  #Fetch the last member
		factorT=TotalFrames/float(TotalMS)

		# Scale the drawn X and Y to scale of the pointer, which is .35 either way.
		# The 200 here is the size of the Canvas in Hansel.  Probably should do
		# a max-min loop, but not sure yet.......
		factorX=0.7/200.0
		factorY=0.35/200.0

		# Get the parms of selected item.
		Xparm=SelItem.ParameterByCode(poser.kParmCodeXTRAN)
		Yparm=SelItem.ParameterByCode(poser.kParmCodeYTRAN)

		# Get the starting point of the record, and make the moves relative
		# to this point.
		(dummyMS,startX,startY,Status)=MainRecord[0]  #Fetch the first member

		for (MS,X,Y,Status) in MainRecord:
			if Status==6: break
			Frame=StartFrame+int(factorT * float(MS))
			scene.SetFrame(Frame-1)
			Xparm.SetValue(factorX*(float(X)-startX)-0.35)
			Yparm.SetValue(0.35-factorY*(float(Y)-startY))

		self.master.destroy()


	# - - - - - - - - - - - - - - - - - -

	def HandleCancel(self):
		self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
