#-----------------------------------------------------------------------------
# Hansel
# Copyright (c) 2004 David G. Drumright (Ockham)
# Oct 9, 2004
#-----------------------------------------------------------------------------

from Tkinter import *
import tkFileDialog
import pickle
import os
# Note the absence of import poser: Hansel really doesn't use Poser at all.

#-----------------------------------------------------------------------------

global FullFileName
global Status
global StartTime
global OldX
global OldY
global imGIF

FullFileName=''
Status=0
StartTime=0
ElapsedTime=0
OldX=0.0
OldY=0.0

MainRecord=[]  # start the main array
# Each element of this array is: (Time,X,Y,Status)
# where Time is in milliseconds from the starting click.

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
	def __init__(self, master, textMessage):
		self.master = master
		master.title("Hansel")

		self.TopFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
		self.TopFrame.grid(row=1,column=0)
		self.PicFrame=Frame(self.master,width=480,height=320,borderwidth=2,relief=RIDGE)
		self.PicFrame.grid(row=2,column=0)
		self.PicFrame.grid_propagate(0)
		self.BotFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
		self.BotFrame.grid(row=3,column=0)

		self.StatusEntry = Entry(self.TopFrame,width=70)
		self.StatusEntry.grid(row=0,column=0,columnspan=2)
		self.StatusEntry.insert(0,"Draw shape.  Right-click to end.")
		self.Clock=Entry(self.TopFrame,width=9)
		self.Clock.grid(row=0,column=2)
		self.Clock.insert(0,str(0.0))

		self.Pic = Canvas(self.PicFrame,  width=300,height=300,bg="white",borderwidth=2,relief=RIDGE)
		self.Pic.grid(row=0,column=0)
		self.Pic.bind("<Button-1>",self.LpressAction)
		self.Pic.bind("<ButtonRelease-1>",self.LreleaseAction)
		self.Pic.bind("<Button-3>",self.RpressAction)
		self.Pic.bind("<Any-Key>",self.RpressAction)
		self.Pic.bind("<Motion>",self.MouseAction)

		# Go/nogo buttons.
		self.ButtonGIF = Button(self.BotFrame, text="Background GIF", command=self.HandleGIF)
		self.ButtonGIF.grid(row=0, column=0)

		self.ButtonName = Button(self.BotFrame, text="Name file", command=self.HandleName)
		self.ButtonName.grid(row=0, column=1)

		self.ButtonCancel = Button(self.BotFrame, text="Cancel", command=self.HandleCancel)
		self.ButtonCancel.grid(row=0, column=2)

		self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

	# - - - - - - - - - - - - - - - - - -

	def ShowStatus(self,S): # Just saving repetition
		self.StatusEntry.delete(0,END)
		self.StatusEntry.insert(0,S)
		self.StatusEntry.update_idletasks()

	# - - - - - - - - - - - - - - - - - -
	# The first Lpress starts everything.  After that, L-release starts a Lift, L-click ends
	# a Lift and starts a Trace.  R-press ends everything.
	# Status variable:
	# 0 = Idle
	# 1 = First Start
	# 2 = Start trace, not first
	# 3 = In trace
	# 4 = Start Lift
	# 5 = In Lift
	# 6 = End
	# Reminder: the receiver must treat 1 as both start of all and starting one trace.
	# - - - - - - - - - - - - - - - - - -

	def LpressAction(self,event):  # At beginning, do special green; otherwise, just mark start of trace
		global OldX,OldY,Status,StartTime,ElapsedTime
		global MainRecord
		if Status==0: # Begin
			# Mark start with a green dot.	Note that we don't use real OldX/OldY here,
			# partly because they will be invalid at this instant, and partly to make a fixed-size arrow.
			# (Could force a temporary Old value, but no reason to.)
			StartTime=event.time  # Set the start time
			ElapsedTime=0  # Set the start time
			self.Pic.create_line(event.x-3,event.y-3,event.x,event.y,fill="green",width=5)
			Status=1 # Mark start of all
			MainRecord.append((0,event.x,event.y,Status))
			Status=3 # In trace for next event
			self.Pic.focus_set() # Grab keyboard
		elif Status < 6: # Starting a trace
			Status=2 # Start trace
			ElapsedTime=event.time-StartTime  # Set the elapsed
			MainRecord.append((ElapsedTime,event.x,event.y,Status))
			Status=3 # In trace for next event
		OldX=event.x
		OldY=event.y

	# - - - - - - - - - - - - - - - - - -

	def LreleaseAction(self,event):
		global OldX,OldY,Status
		global MainRecord
		# Mark start of Lift.
		if Status>0:
			Status=4  # Start of lift
			MainRecord.append((ElapsedTime,event.x,event.y,Status))
			Status=5  # Leave at In-lift for next event
		OldX=event.x
		OldY=event.y

	# - - - - - - - - - - - - - - - - - -

	def MouseAction(self,event):
		# Receives mousemoves
		global OldX,OldY,Status
		global MainRecord
		if Status == 0: return

		if Status==3: # If in trace:
			self.Pic.create_line(OldX,OldY,event.x,event.y,fill="gold",width=2)
			MainRecord.append((ElapsedTime,event.x,event.y,Status))
		elif Status==5: # If in lift: just record.
			MainRecord.append((ElapsedTime,event.x,event.y,Status))
		# No action at all in other status.
		OldX=event.x
		OldY=event.y

	# - - - - - - - - - - - - - - - - - -

	def RpressAction(self,event):
		global OldX,OldY,Status
		global MainRecord
		# Rpress does only one thing: ends the action.
		if Status>0:
			# Mark end with a red dot
			self.Pic.create_line(event.x-2,event.y-2,event.x,event.y,fill="red",width=5)
			Status=6 # end all
			MainRecord.append((ElapsedTime,event.x,event.y,Status))
		OldX=event.x
		OldY=event.y

	# - - - - - - - - - - - - - - - - - -

	def HandleName(self):
		global FullFileName
		# Pop up file box to get name.
		BasePathUncut=poser.AppLocation()
		BasePath=BasePathUncut[:(-len('poser.exe'))]
		MyPath=os.path.normpath(BasePath+'/runtime/*.DAT') # Use this for release
		#fn=tkFileDialog.asksaveasfilename(initialfile=MyPath, filetypes=[("DAT file", "*.DAT")] )
		fn=tkFileDialog.asksaveasfilename(initialfile=MyPath, defaultextension=[("DAT file", "*.DAT *.dat")] )

		if not fn:
			self.ShowStatus("No file selected. Try again.")
			return 0

		# Open the file
		FullFileName=os.path.abspath(fn)
		fp=open(FullFileName,'w')
		pickle.dump(MainRecord,fp)
		fp.close()
		self.master.destroy()


	# - - - - - - - - - - - - - - - - - -

	def HandleGIF(self):
		global imGIF
		BasePathUncut=poser.AppLocation()
		BasePath=BasePathUncut[:(-len('poser.exe'))]
		MyPath=os.path.normpath(BasePath+'/runtime/*.GIF') # Use this for release
		fn=tkFileDialog.askopenfilename(initialfile=MyPath, defaultextension=[("GIF file", "*.GIF *.gif")] )

		if not fn:
			self.ShowStatus("No file selected. Try again.")
			return 0
		GIFfile=os.path.normpath(fn)
		imGIF=PhotoImage(file=GIFfile)
		self.Pic.create_image(0,0,anchor=NW,image=imGIF)
		self.Pic.update_idletasks()


	# - - - - - - - - - - - - - - - - - -
	# Unlike the usual, we don't bother the Scene here.
	def Update(self):
		global ElapsedTime
		if (Status > 0) and (Status < 6):
			self.Clock.delete(0,END)
			self.Clock.insert(0,"%6.1f" % (float(ElapsedTime)/1000.0) )
			self.Clock.update_idletasks()
		root.lift()
		ElapsedTime=ElapsedTime+100
		root.after(100, self.Update) # This serves to move the clock
	# - - - - - - - - - - - - - - - - - -

	def HandleCancel(self):
		self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
